@@ -16,6 +16,7 @@ impl Default for super::CommandState {
             stage_infos: Default::default(),
             storage_buffer_length_map: Default::default(),
             work_group_memory_sizes: Vec::new(),
+            push_constants: Vec::new(),
         }
     }
 }
@@ -61,6 +62,7 @@ impl super::CommandState {
         self.stage_infos.fs.clear();
         self.stage_infos.cs.clear();
         self.work_group_memory_sizes.clear();
+        self.push_constants.clear();
     }
 
     fn make_sizes_buffer_update<'a>(
@@ -100,16 +102,29 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
 
         Ok(())
     }
+
     unsafe fn discard_encoding(&mut self) {
         self.leave_blit();
+        // when discarding, we don't have a guarantee that
+        // everything is in a good state, so check carefully
+        if let Some(encoder) = self.state.render.take() {
+            encoder.end_encoding();
+        }
+        if let Some(encoder) = self.state.compute.take() {
+            encoder.end_encoding();
+        }
         self.raw_cmd_buf = None;
     }
+
     unsafe fn end_encoding(&mut self) -> Result<super::CommandBuffer, crate::DeviceError> {
         self.leave_blit();
+        assert!(self.state.render.is_none());
+        assert!(self.state.compute.is_none());
         Ok(super::CommandBuffer {
             raw: self.raw_cmd_buf.take().unwrap(),
         })
     }
+
     unsafe fn reset_all<I>(&mut self, _cmd_bufs: I)
     where
         I: Iterator<Item = super::CommandBuffer>,
@@ -134,74 +149,6 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         encoder.fill_buffer(&buffer.raw, conv::map_range(&range), 0);
     }
 
-    unsafe fn clear_texture(
-        &mut self,
-        texture: &super::Texture,
-        subresource_range: &wgt::ImageSubresourceRange,
-    ) {
-        let shared = self.shared.clone();
-        let encoder = self.enter_blit();
-
-        let format_desc = texture.format.describe();
-
-        let mip_range = subresource_range.base_mip_level..match subresource_range.mip_level_count {
-            Some(c) => subresource_range.base_mip_level + c.get(),
-            None => texture.mip_levels,
-        };
-        let array_range = subresource_range.base_array_layer
-            ..match subresource_range.array_layer_count {
-                Some(c) => subresource_range.base_array_layer + c.get(),
-                None => texture.array_layers,
-            };
-
-        for mip_level in mip_range {
-            // Note that Metal requires this only to be a multiple of the pixel size, not some other constant like in other APIs.
-            let mip_size = texture.copy_size.at_mip_level(mip_level);
-            let bytes_per_row = mip_size.width as u64 / format_desc.block_dimensions.0 as u64
-                * format_desc.block_size as u64;
-            let max_rows_per_copy = super::ZERO_BUFFER_SIZE / bytes_per_row;
-            // round down to a multiple of rows needed by the texture format
-            let max_rows_per_copy = max_rows_per_copy / format_desc.block_dimensions.1 as u64
-                * format_desc.block_dimensions.1 as u64;
-            assert!(max_rows_per_copy > 0, "Zero buffer size is too small to fill a single row of a texture of type {:?}, size {:?} and format {:?}",
-                        texture.raw_type, texture.copy_size, texture.format);
-
-            for array_layer in array_range.clone() {
-                // 3D textures are quickly massive in memory size, so we don't bother trying to do more than one layer at once.
-                for z in 0..mip_size.depth as u64 {
-                    // May need multiple copies for each subresource! We assume that we never need to split a row.
-                    let mut num_rows_left = mip_size.height as u64;
-                    while num_rows_left > 0 {
-                        let num_rows = num_rows_left.min(max_rows_per_copy);
-                        let source_size = mtl::MTLSize {
-                            width: mip_size.width as u64,
-                            height: num_rows,
-                            depth: 1,
-                        };
-                        let destination_origion = mtl::MTLOrigin {
-                            x: 0,
-                            y: mip_size.height as u64 - num_rows_left,
-                            z,
-                        };
-                        encoder.copy_from_buffer_to_texture(
-                            &shared.zero_buffer,
-                            0,
-                            bytes_per_row,
-                            bytes_per_row * num_rows,
-                            source_size,
-                            &texture.raw,
-                            array_layer as u64,
-                            mip_level as u64,
-                            destination_origion,
-                            mtl::MTLBlitOption::empty(),
-                        );
-                        num_rows_left -= num_rows;
-                    }
-                }
-            }
-        }
-    }
-
     unsafe fn copy_buffer_to_buffer<T>(
         &mut self,
         src: &super::Buffer,
@@ -642,12 +589,41 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
 
     unsafe fn set_push_constants(
         &mut self,
-        _layout: &super::PipelineLayout,
-        _stages: wgt::ShaderStages,
-        _offset: u32,
-        _data: &[u32],
+        layout: &super::PipelineLayout,
+        stages: wgt::ShaderStages,
+        offset: u32,
+        data: &[u32],
     ) {
-        //TODO
+        let state_pc = &mut self.state.push_constants;
+        if state_pc.len() < layout.total_push_constants as usize {
+            state_pc.resize(layout.total_push_constants as usize, 0);
+        }
+        assert_eq!(offset as usize % WORD_SIZE, 0);
+
+        let offset = offset as usize / WORD_SIZE;
+        state_pc[offset..offset + data.len()].copy_from_slice(data);
+
+        if stages.contains(wgt::ShaderStages::COMPUTE) {
+            self.state.compute.as_ref().unwrap().set_bytes(
+                layout.push_constants_infos.cs.unwrap().buffer_index as _,
+                (layout.total_push_constants as usize * WORD_SIZE) as _,
+                state_pc.as_ptr() as _,
+            )
+        }
+        if stages.contains(wgt::ShaderStages::VERTEX) {
+            self.state.render.as_ref().unwrap().set_vertex_bytes(
+                layout.push_constants_infos.vs.unwrap().buffer_index as _,
+                (layout.total_push_constants as usize * WORD_SIZE) as _,
+                state_pc.as_ptr() as _,
+            )
+        }
+        if stages.contains(wgt::ShaderStages::FRAGMENT) {
+            self.state.render.as_ref().unwrap().set_fragment_bytes(
+                layout.push_constants_infos.fs.unwrap().buffer_index as _,
+                (layout.total_push_constants as usize * WORD_SIZE) as _,
+                state_pc.as_ptr() as _,
+            )
+        }
     }
 
     unsafe fn insert_debug_marker(&mut self, label: &str) {
